
#include "stdafx.h"
#include "GlobalTools.h"


G31DDC_OPEN_DEVICE					G31_OpenDevice;
G31DDC_CLOSE_DEVICE					G31_CloseDevice;
G31DDC_SET_POWER					G31_SetPower;
G31DDC_GET_POWER					G31_GetPower;
G31DDC_START_IF						G31_StartIF;
G31DDC_SET_DDC1						G31_SetDDC1;
G31DDC_GET_DDC1						G31_GetDDC1;
G31DDC_GET_DDC2						G31_GetDDC2;
G31DDC_SET_CALLBACKS				G31_SetCallbacks;
G31DDC_SET_FREQUENCY				G31_SetFrequency;
G31DDC_GET_FREQUENCY				G31_GetFrequency;
G31DDC_START_DDC1					G31_StartDDC1;
G31DDC_STOP_DDC1					G31_StopDDC1;
G31DDC_START_DDC2					G31_StartDDC2;
G31DDC_STOP_DDC2					G31_StopDDC2;
G31DDC_START_AUDIO					G31_StartAudio;
G31DDC_STOP_AUDIO					G31_StopAudio;
G31DDC_SET_AGC						G31_SetAGC;
G31DDC_SET_AGC_PARAMS				G31_SetAGCParams;
G31DDC_SET_LED						G31_SetLED;
G31DDC_SET_MW_FILTER				G31_SetMWFilter;
G31DDC_SET_ATTENUATOR				G31_SetAttenuator;
G31DDC_SET_DITHERING				G31_SetDithering;
G31DDC_SET_DEMODULATOR_MODE			G31_SetDemodulatorMode;
G31DDC_SET_DDC1_FREQUENCY			G31_SetDDC1Frequency;
G31DDC_SET_DDC2_FREQUENCY			G31_SetDDC2Frequency;
G31DDC_GET_DDC1_FREQUENCY			G31_GetDDC1Frequency;
G31DDC_GET_DDC2_FREQUENCY			G31_GetDDC2Frequency;
G31DDC_SET_ADC_NOISE_BLANKER		G31_SetADCNoiseBlanker;
G31DDC_GET_DEMODULATOR_FREQUENCY	G31_GetDemodulatorFrequency;
G31DDC_SET_DEMODULATOR_FREQUENCY	G31_SetDemodulatorFrequency;
G31DDC_SET_DEMODULATOR_FILTER_BANDWIDTH G31_SetDemodulatorFilterBandwidth;


G31DDCDll::G31DDCDll()
{
	hDevice=-1;
	h_lib=NULL;
	IQ_freq=200000;
	IQ_spow=13;
	DDCType=9;
	IsRunning=0;
	DataBuf = NULL;
	dBcorr=1.f;
	lastMode=-1;
}

G31DDCDll::~G31DDCDll()
{
	CloseExtRx();
}

//void extIOCallback(int cnt, int status, float IQoffs, short IQdata[])
typedef void (*DLL_extIOCallback)(int, int, float, unsigned char *);
extern DLL_extIOCallback extIOCallback;
void __stdcall DDC1StreamCallback(CONST VOID *Buffer,UINT32 NumberOfSamples,UINT32 BitsPerSample,DWORD* UserData)
{
	G31DDCDll *dll=(G31DDCDll*)UserData;
	if(dll->DataBuf)
		{
		int *iq_buf=(int*)Buffer;
		for(UINT32 i=0; i<(NumberOfSamples<<1); i++)
			{
			dll->DataBuf[i]=(float)iq_buf[i]*dll->dBcorr; //dBOffset dB
			}
		extIOCallback(NumberOfSamples<<1,0,0,(unsigned char*)dll->DataBuf);
		}
}

#include "math.h"
extern UINT LastSerNum;
BOOL G31DDCDll::OpenExtRx()
{	
	if(!LoadDLL()) goto err;

	G31_SetDDC1(hDevice,0);

    //retrieve current DDC type of the DDC1
    G31_GetDDC1(hDevice,&DDCTypeIndex,&DDCInfo);

	//set Parameter
	_SetATT(myPara.ATT);
	_SetDither(myPara.DIT);
	_SetNB(myPara.NB);

	//only for Winrad, HDSDR ... or clone
	if(!myPara.msgHWND)	
		{
		dBcorr=(float)pow(10,(double)(myPara.dBOffset)*.05);  //.05 = log20;
		}
	else dBcorr=1.f;

	 //IFCallbacks 
	memset(&Callbacks,0,sizeof(G31DDC_CALLBACKS));
    Callbacks.DDC1StreamCallback=DDC1StreamCallback;
	lastMode=-1;
	return SetSampleMode(myPara.mode);

err:
	FreeDLL();
	return FALSE;
}

void G31DDCDll::CloseExtRx()
{
	if(hDevice>=0)   
		{
		CodecStop();
		G31_SetPower(hDevice,FALSE);
		G31_SetLED(hDevice,0);
		}
	FreeDLL();
}

void G31DDCDll::Set_dBmOfsset()
{
	if(!myPara.msgHWND)	
		{
		dBcorr=(float)pow(10,(double)(myPara.dBOffset)*.05);  //.05 = log20;
		}
	else dBcorr=1.f;
}


BOOL G31DDCDll::LoadDLL()
{
	h_lib = LoadLibrary("G31DDCAPI.dll");
	if (h_lib==NULL) 
		{
		return FALSE;
		}

    //retrieve addresses of used API functions
    if(!(G31_OpenDevice=(G31DDC_OPEN_DEVICE)GetProcAddress(h_lib,"OpenDevice"))) goto err;
    if(!(G31_CloseDevice=(G31DDC_CLOSE_DEVICE)GetProcAddress(h_lib,"CloseDevice"))) goto err;
    if(!(G31_SetPower=(G31DDC_SET_POWER)GetProcAddress(h_lib,"SetPower"))) goto err;
    if(!(G31_GetPower=(G31DDC_GET_POWER)GetProcAddress(h_lib,"GetPower"))) goto err;
	if(!(G31_StartIF=(G31DDC_START_IF)GetProcAddress(h_lib,"StartIF"))) goto err;
    if(!(G31_SetDDC1=(G31DDC_SET_DDC1)GetProcAddress(h_lib,"SetDDC1"))) goto err;
    if(!(G31_GetDDC1=(G31DDC_GET_DDC1)GetProcAddress(h_lib,"GetDDC1"))) goto err;
    if(!(G31_GetDDC2=(G31DDC_GET_DDC2)GetProcAddress(h_lib,"GetDDC2"))) goto err;    
    if(!(G31_SetCallbacks=(G31DDC_SET_CALLBACKS)GetProcAddress(h_lib,"SetCallbacks"))) goto err;
    if(!(G31_SetFrequency=(G31DDC_SET_FREQUENCY)GetProcAddress(h_lib,"SetFrequency"))) goto err;
    if(!(G31_GetFrequency=(G31DDC_GET_FREQUENCY)GetProcAddress(h_lib,"GetFrequency"))) goto err;
    if(!(G31_StartDDC1=(G31DDC_START_DDC1)GetProcAddress(h_lib,"StartDDC1"))) goto err;
    if(!(G31_StopDDC1=(G31DDC_STOP_DDC1)GetProcAddress(h_lib,"StopDDC1"))) goto err;
    if(!(G31_StartDDC2=(G31DDC_START_DDC2)GetProcAddress(h_lib,"StartDDC2"))) goto err;
    if(!(G31_StopDDC2=(G31DDC_STOP_DDC2)GetProcAddress(h_lib,"StopDDC2"))) goto err;
    if(!(G31_StartAudio=(G31DDC_START_AUDIO)GetProcAddress(h_lib,"StartAudio"))) goto err;
    if(!(G31_StopAudio=(G31DDC_STOP_AUDIO)GetProcAddress(h_lib,"StopAudio"))) goto err;
    if(!(G31_SetAGC=(G31DDC_SET_AGC)GetProcAddress(h_lib,"SetAGC"))) goto err;
	if(!(G31_SetLED=(G31DDC_SET_LED)GetProcAddress(h_lib,"SetLED"))) goto err;
	if(!(G31_SetMWFilter=(G31DDC_SET_MW_FILTER)GetProcAddress(h_lib,"SetMWFilter"))) goto err;
	if(!(G31_SetAttenuator=(G31DDC_SET_ATTENUATOR)GetProcAddress(h_lib,"SetAttenuator"))) goto err;
	if(!(G31_SetDithering=(G31DDC_SET_DITHERING)GetProcAddress(h_lib,"SetDithering"))) goto err;
    if(!(G31_SetAGCParams=(G31DDC_SET_AGC_PARAMS)GetProcAddress(h_lib,"SetAGCParams"))) goto err;
    if(!(G31_SetDemodulatorMode=(G31DDC_SET_DEMODULATOR_MODE)GetProcAddress(h_lib,"SetDemodulatorMode"))) goto err;
    if(!(G31_SetDemodulatorFilterBandwidth=(G31DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)GetProcAddress(h_lib,"SetDemodulatorFilterBandwidth"))) goto err;
    if(!(G31_SetDDC1Frequency=(G31DDC_SET_DDC1_FREQUENCY)GetProcAddress(h_lib,"SetDDC1Frequency"))) goto err;
    if(!(G31_SetDDC2Frequency=(G31DDC_SET_DDC2_FREQUENCY)GetProcAddress(h_lib,"SetDDC2Frequency"))) goto err;
    if(!(G31_SetDemodulatorFrequency=(G31DDC_SET_DEMODULATOR_FREQUENCY)GetProcAddress(h_lib,"SetDemodulatorFrequency"))) goto err;
    if(!(G31_GetDDC1Frequency=(G31DDC_GET_DDC1_FREQUENCY)GetProcAddress(h_lib,"GetDDC1Frequency"))) goto err;
    if(!(G31_GetDDC2Frequency=(G31DDC_GET_DDC2_FREQUENCY)GetProcAddress(h_lib,"GetDDC2Frequency"))) goto err;
    if(!(G31_GetDemodulatorFrequency=(G31DDC_GET_DEMODULATOR_FREQUENCY)GetProcAddress(h_lib,"GetDemodulatorFrequency"))) goto err;
    if(!(G31_SetADCNoiseBlanker=(G31DDC_SET_ADC_NOISE_BLANKER)GetProcAddress(h_lib,"SetADCNoiseBlanker"))) goto err;

    //open the first available G31DDC device
	hDevice=G31_OpenDevice(G3XDDC_OPEN_FIRST);
    if(hDevice>=0) return TRUE;

err:
	return FALSE;
}

void G31DDCDll::ClearVariables()
{
	h_lib=NULL;
	hDevice=-1;
	G31_OpenDevice=NULL;
	G31_CloseDevice=NULL;
	G31_SetPower=NULL;
	G31_GetPower=NULL;
	G31_StartIF=NULL;
	G31_SetDDC1=NULL;
	G31_GetDDC1=NULL;
	G31_GetDDC2=NULL;
	G31_SetCallbacks=NULL;
	G31_SetFrequency=NULL;
	G31_GetFrequency=NULL;
	G31_StartDDC1=NULL;
	G31_StopDDC1=NULL;
	G31_StartDDC2=NULL;
	G31_StopDDC2=NULL;
	G31_StartAudio=NULL;
	G31_StopAudio=NULL;
	G31_SetAGC=NULL;
	G31_SetLED=NULL;
	G31_SetMWFilter=NULL;
	G31_SetAttenuator=NULL;
	G31_SetDithering=NULL;
	G31_SetAGCParams=NULL;
	G31_SetDemodulatorMode=NULL;
	G31_SetDDC1Frequency=NULL;
	G31_SetDDC2Frequency=NULL;
	G31_GetDDC1Frequency=NULL;
	G31_GetDDC2Frequency=NULL;
	G31_SetADCNoiseBlanker=NULL;
	G31_GetDemodulatorFrequency=NULL;
	G31_SetDemodulatorFrequency=NULL;
	G31_SetDemodulatorFilterBandwidth=NULL;
	if(DataBuf)	{delete []DataBuf;	DataBuf = NULL;}
}

BOOL G31DDCDll::FreeDLL(void)
{
	if(h_lib)
		{
		if(hDevice>=0) G31_CloseDevice(hDevice);
		FreeLibrary(h_lib);
		ClearVariables();
		return TRUE;
		}
	return FALSE;
}

BOOL G31DDCDll::CodecStop()
{
	IsRunning=0;
	if(hDevice<0) return FALSE;
	G31_StopDDC1(hDevice);
    return G31_SetCallbacks(hDevice,NULL,0);
}

BOOL G31DDCDll::CodecStart()
{
	// configure the device 
	if(hDevice<0) return FALSE;

	SetPower(TRUE);
	G31_SetDDC1(hDevice,DDCType);	// to get the stream
	G31_SetMWFilter(hDevice,FALSE);
    G31_SetCallbacks(hDevice,&Callbacks,(DWORD*)this);
	if(!G31_StartDDC1(hDevice,1<<IQ_spow)) return FALSE;
	IsRunning=1;
	return TRUE;
}

BOOL G31DDCDll::SetSampleMode(int mode)
{
	if(hDevice<0) return FALSE;
	if(mode>4) mode=4;
	if(mode<0) mode=0;
	if(lastMode==mode) return TRUE;
	int	bIsRunning=IsRunning;
	if(bIsRunning) CodecStop();

	myPara.mode=mode;
	switch(myPara.mode)
		{
		case	0:	 IQ_freq=50000;		IQ_spow=11;	 	DDCType=3;	break; //SampleRate=     50000,Bandwidth=    40000,BitsPerSample=32
		case	1:	 IQ_freq=100000;	IQ_spow=12;		DDCType=6;	break; //SampleRate=    100000,Bandwidth=    80000,BitsPerSample=32
		default:	 myPara.mode=2;
		case	2:	 IQ_freq=200000;	IQ_spow=13;		DDCType=9;	break; //SampleRate=    200000,Bandwidth=   160000,BitsPerSample=32
		case	3:	 IQ_freq=500000;	IQ_spow=14;		DDCType=13;	break; //SampleRate=    500000,Bandwidth=   400000,BitsPerSample=32
		case	4:	 IQ_freq=1000000;	IQ_spow=14;		DDCType=16;	break; //SampleRate=   1000000,Bandwidth=   800000,BitsPerSample=32
		}

	IQ_spow=12;

	if(!DataBuf)
		{
		DataBuf		= new float[(1<<(IQ_spow+1))];
		}

	if(extIOCallback) 
		{
		extIOCallback(-1,100,0,0); // tells the ExtIQ-APP, that the sampling frequency was changed
		}

	if(bIsRunning) CodecStart();
	return TRUE;
}

int G31DDCDll::SetFrequency(int freq)
{
	if(freq<10) freq=10;
	if(freq>50000000) freq=50000000;
	lastFrq=freq;
	if(hDevice>=0) return G31_SetFrequency(hDevice,0,freq);	
	return freq;
}

void G31DDCDll::SetPower(BOOL onoff)
{
	if (hDevice>=0)
		{
		int status=0;
		G31_GetPower(hDevice,&status);
		if(onoff)
			{
			if(!status) G31_SetPower(hDevice,1);
			status=1; 
			}
		else
			{
			if(status) G31_SetPower(hDevice,0);
			status=3;
			}
		G31_SetLED(hDevice,status); 
		}
}

void G31DDCDll::_SetATT(int att)
{
	//Attenuator value in dB. Permitted values are: 0,3,6,9,12,15,18,21. 
	if (hDevice>=0)
		{
		myPara.ATT=att;
		if(att)	G31_SetAttenuator(hDevice,21);
		else	G31_SetAttenuator(hDevice,0);
		}
}

void G31DDCDll::_SetDither(BOOL enable)
{
	if (hDevice>=0)	
		{
		myPara.DIT=enable;
		G31_SetDithering(hDevice,enable);
		}
}

void G31DDCDll::_SetNB(BOOL enable)
{
	if (hDevice>=0)	
		{
		myPara.NB=enable;
		G31_SetADCNoiseBlanker(hDevice,enable);
		}
}
