
#include "stdafx.h"
#include "GlobalTools.h"


G33DDC_OPEN_DEVICE					G33_OpenDevice;
G33DDC_CLOSE_DEVICE					G33_CloseDevice;
G33DDC_SET_POWER					G33_SetPower;
G33DDC_GET_POWER					G33_GetPower;
G33DDC_START_IF						G33_StartIF;
G33DDC_SET_DDC1						G33_SetDDC1;
G33DDC_GET_DDC1						G33_GetDDC1;
G33DDC_GET_DDC2						G33_GetDDC2;
G33DDC_SET_CALLBACKS				G33_SetCallbacks;
G33DDC_SET_FREQUENCY				G33_SetFrequency;
G33DDC_GET_FREQUENCY				G33_GetFrequency;
G33DDC_START_DDC1					G33_StartDDC1;
G33DDC_STOP_DDC1					G33_StopDDC1;
G33DDC_START_DDC2					G33_StartDDC2;
G33DDC_STOP_DDC2					G33_StopDDC2;
G33DDC_START_AUDIO					G33_StartAudio;
G33DDC_STOP_AUDIO					G33_StopAudio;
G33DDC_SET_AGC						G33_SetAGC;
G33DDC_SET_AGC_PARAMS				G33_SetAGCParams;
G33DDC_SET_LED						G33_SetLED;
G33DDC_SET_PREAMP					G33_SetPreamp;
G33DDC_SET_PRESELECTORS				G33_SetPreselectors;
G33DDC_SET_ATTENUATOR				G33_SetAttenuator;
G33DDC_SET_DITHERING				G33_SetDithering;
G33DDC_SET_DEMODULATOR_MODE			G33_SetDemodulatorMode;
G33DDC_SET_DDC1_FREQUENCY			G33_SetDDC1Frequency;
G33DDC_SET_DDC2_FREQUENCY			G33_SetDDC2Frequency;
G33DDC_GET_DDC1_FREQUENCY			G33_GetDDC1Frequency;
G33DDC_GET_DDC2_FREQUENCY			G33_GetDDC2Frequency;
G33DDC_SET_ADC_NOISE_BLANKER		G33_SetADCNoiseBlanker;
G33DDC_GET_DEMODULATOR_FREQUENCY	G33_GetDemodulatorFrequency;
G33DDC_SET_DEMODULATOR_FREQUENCY	G33_SetDemodulatorFrequency;
G33DDC_SET_DEMODULATOR_FILTER_BANDWIDTH G33_SetDemodulatorFilterBandwidth;


G33DDCDll::G33DDCDll()
{
	hDevice=-1;
	h_lib=NULL;
	IQ_freq=200000;
	IQ_spow=13;
	DDCType=9;
	IsRunning=0;
	DataBuf = NULL;
	dBcorr=1.f;
	lastMode=lastLow=lastHigh-1;
}

G33DDCDll::~G33DDCDll()
{
	CloseExtRx();
}

//void extIOCallback(int cnt, int status, float IQoffs, short IQdata[])
typedef void (*DLL_extIOCallback)(int, int, float, unsigned char *);
extern DLL_extIOCallback extIOCallback;
void __stdcall DDC1StreamCallback(CONST VOID *Buffer,UINT32 NumberOfSamples,UINT32 BitsPerSample,DWORD* UserData)
{
	G33DDCDll *dll=(G33DDCDll*)UserData;
	if(dll->DataBuf)
		{
		int *iq_buf=(int*)Buffer;
		for(UINT32 i=0; i<(NumberOfSamples<<1); i++)
			{
			dll->DataBuf[i]=(float)iq_buf[i]*dll->dBcorr; //dBOffset dB
			}
		extIOCallback(NumberOfSamples<<1,0,0,(unsigned char*)dll->DataBuf);
		}
}

#include "math.h"
extern UINT LastSerNum;
BOOL G33DDCDll::OpenExtRx()
{	
	if(!LoadDLL()) goto err;

	G33_SetDDC1(hDevice,0);

    //retrieve current DDC type of the DDC1
    G33_GetDDC1(hDevice,&DDCTypeIndex,&DDCInfo);

	//set Parameter
	_SetATT(myPara.ATT);
	_SetDither(myPara.DIT);
	_SetNB(myPara.NB);
	_SetPRE(myPara.PRE);

	//only for Winrad, HDSDR ... or clone
	if(!myPara.msgHWND)	
		{
		dBcorr=(float)pow(10,(double)(myPara.dBOffset)*.05);  //.05 = log20;
		}
	else dBcorr=1.f;

	 //IFCallbacks 
	memset(&Callbacks,0,sizeof(G33DDC_CALLBACKS));
    Callbacks.DDC1StreamCallback=DDC1StreamCallback;
	lastMode=-1;
	return SetSampleMode(myPara.mode);

err:
	FreeDLL();
	return FALSE;
}

void G33DDCDll::CloseExtRx()
{
	if(hDevice>=0)   
		{
		CodecStop();
		G33_SetPower(hDevice,FALSE);
		G33_SetLED(hDevice,0);
		}
	FreeDLL();
}

void G33DDCDll::Set_dBmOfsset()
{
	if(!myPara.msgHWND)	
		{
		dBcorr=(float)pow(10,(double)(myPara.dBOffset)*.05);  //.05 = log20;
		}
	else dBcorr=1.f;
}


BOOL G33DDCDll::LoadDLL()
{
	h_lib = LoadLibrary("G33DDCAPI.dll");
	if (h_lib==NULL) return FALSE;

    //retrieve addresses of used API functions
    if(!(G33_OpenDevice=(G33DDC_OPEN_DEVICE)GetProcAddress(h_lib,"OpenDevice"))) goto err;
    if(!(G33_CloseDevice=(G33DDC_CLOSE_DEVICE)GetProcAddress(h_lib,"CloseDevice"))) goto err;
    if(!(G33_SetPower=(G33DDC_SET_POWER)GetProcAddress(h_lib,"SetPower"))) goto err;
    if(!(G33_GetPower=(G33DDC_GET_POWER)GetProcAddress(h_lib,"GetPower"))) goto err;
	if(!(G33_StartIF=(G33DDC_START_IF)GetProcAddress(h_lib,"StartIF"))) goto err;
    if(!(G33_SetDDC1=(G33DDC_SET_DDC1)GetProcAddress(h_lib,"SetDDC1"))) goto err;
    if(!(G33_GetDDC1=(G33DDC_GET_DDC1)GetProcAddress(h_lib,"GetDDC1"))) goto err;
    if(!(G33_GetDDC2=(G33DDC_GET_DDC2)GetProcAddress(h_lib,"GetDDC2"))) goto err;    
    if(!(G33_SetCallbacks=(G33DDC_SET_CALLBACKS)GetProcAddress(h_lib,"SetCallbacks"))) goto err;
    if(!(G33_SetFrequency=(G33DDC_SET_FREQUENCY)GetProcAddress(h_lib,"SetFrequency"))) goto err;
    if(!(G33_GetFrequency=(G33DDC_GET_FREQUENCY)GetProcAddress(h_lib,"GetFrequency"))) goto err;
    if(!(G33_StartDDC1=(G33DDC_START_DDC1)GetProcAddress(h_lib,"StartDDC1"))) goto err;
    if(!(G33_StopDDC1=(G33DDC_STOP_DDC1)GetProcAddress(h_lib,"StopDDC1"))) goto err;
    if(!(G33_StartDDC2=(G33DDC_START_DDC2)GetProcAddress(h_lib,"StartDDC2"))) goto err;
    if(!(G33_StopDDC2=(G33DDC_STOP_DDC2)GetProcAddress(h_lib,"StopDDC2"))) goto err;
    if(!(G33_StartAudio=(G33DDC_START_AUDIO)GetProcAddress(h_lib,"StartAudio"))) goto err;
    if(!(G33_StopAudio=(G33DDC_STOP_AUDIO)GetProcAddress(h_lib,"StopAudio"))) goto err;
    if(!(G33_SetAGC=(G33DDC_SET_AGC)GetProcAddress(h_lib,"SetAGC"))) goto err;
	if(!(G33_SetLED=(G33DDC_SET_LED)GetProcAddress(h_lib,"SetLED"))) goto err;
	if(!(G33_SetPreamp=(G33DDC_SET_PREAMP)GetProcAddress(h_lib,"SetPreamp"))) goto err;
	if(!(G33_SetPreselectors=(G33DDC_SET_PRESELECTORS)GetProcAddress(h_lib,"SetPreselectors"))) goto err;
	if(!(G33_SetAttenuator=(G33DDC_SET_ATTENUATOR)GetProcAddress(h_lib,"SetAttenuator"))) goto err;
	if(!(G33_SetDithering=(G33DDC_SET_DITHERING)GetProcAddress(h_lib,"SetDithering"))) goto err;
    if(!(G33_SetAGCParams=(G33DDC_SET_AGC_PARAMS)GetProcAddress(h_lib,"SetAGCParams"))) goto err;
    if(!(G33_SetDemodulatorMode=(G33DDC_SET_DEMODULATOR_MODE)GetProcAddress(h_lib,"SetDemodulatorMode"))) goto err;
    if(!(G33_SetDemodulatorFilterBandwidth=(G33DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)GetProcAddress(h_lib,"SetDemodulatorFilterBandwidth"))) goto err;
    if(!(G33_SetDDC1Frequency=(G33DDC_SET_DDC1_FREQUENCY)GetProcAddress(h_lib,"SetDDC1Frequency"))) goto err;
    if(!(G33_SetDDC2Frequency=(G33DDC_SET_DDC2_FREQUENCY)GetProcAddress(h_lib,"SetDDC2Frequency"))) goto err;
    if(!(G33_SetDemodulatorFrequency=(G33DDC_SET_DEMODULATOR_FREQUENCY)GetProcAddress(h_lib,"SetDemodulatorFrequency"))) goto err;
    if(!(G33_GetDDC1Frequency=(G33DDC_GET_DDC1_FREQUENCY)GetProcAddress(h_lib,"GetDDC1Frequency"))) goto err;
    if(!(G33_GetDDC2Frequency=(G33DDC_GET_DDC2_FREQUENCY)GetProcAddress(h_lib,"GetDDC2Frequency"))) goto err;
    if(!(G33_GetDemodulatorFrequency=(G33DDC_GET_DEMODULATOR_FREQUENCY)GetProcAddress(h_lib,"GetDemodulatorFrequency"))) goto err;
    if(!(G33_SetADCNoiseBlanker=(G33DDC_SET_ADC_NOISE_BLANKER)GetProcAddress(h_lib,"SetADCNoiseBlanker"))) goto err;

    //open the first available G33DDC device
	hDevice=G33_OpenDevice(G3XDDC_OPEN_FIRST);
    if(hDevice>=0) return TRUE;

err:
	return FALSE;
}

void G33DDCDll::ClearVariables()
{
	h_lib=NULL;
	hDevice=-1;
	G33_OpenDevice=NULL;
	G33_CloseDevice=NULL;
	G33_SetPower=NULL;
	G33_GetPower=NULL;
	G33_StartIF=NULL;
	G33_SetDDC1=NULL;
	G33_GetDDC1=NULL;
	G33_GetDDC2=NULL;
	G33_SetCallbacks=NULL;
	G33_SetFrequency=NULL;
	G33_GetFrequency=NULL;
	G33_StartDDC1=NULL;
	G33_StopDDC1=NULL;
	G33_StartDDC2=NULL;
	G33_StopDDC2=NULL;
	G33_StartAudio=NULL;
	G33_StopAudio=NULL;
	G33_SetAGC=NULL;
	G33_SetLED=NULL;
	G33_SetPreamp=NULL;
	G33_SetPreselectors=NULL;
	G33_SetAttenuator=NULL;
	G33_SetDithering=NULL;
	G33_SetAGCParams=NULL;
	G33_SetDemodulatorMode=NULL;
	G33_SetDDC1Frequency=NULL;
	G33_SetDDC2Frequency=NULL;
	G33_GetDDC1Frequency=NULL;
	G33_GetDDC2Frequency=NULL;
	G33_SetADCNoiseBlanker=NULL;
	G33_GetDemodulatorFrequency=NULL;
	G33_SetDemodulatorFrequency=NULL;
	G33_SetDemodulatorFilterBandwidth=NULL;
	if(DataBuf)	{delete []DataBuf;	DataBuf = NULL;}
}

BOOL G33DDCDll::FreeDLL(void)
{
	if(h_lib)
		{
		if(hDevice>=0) G33_CloseDevice(hDevice);
		FreeLibrary(h_lib);
		ClearVariables();
		return TRUE;
		}
	return FALSE;
}

BOOL G33DDCDll::CodecStop()
{
	IsRunning=0;
	if(hDevice<0) return FALSE;
	G33_StopDDC1(hDevice);
    return G33_SetCallbacks(hDevice,NULL,0);
}

BOOL G33DDCDll::CodecStart()
{
	// configure the device 
	if(hDevice<0) return FALSE;

	SetPower(TRUE);
	G33_SetDDC1(hDevice,DDCType);	// to get the stream
    G33_SetCallbacks(hDevice,&Callbacks,(DWORD*)this);
	if(!G33_StartDDC1(hDevice,1<<IQ_spow)) return FALSE;
	IsRunning=1;
	return TRUE;
}

BOOL G33DDCDll::SetSampleMode(int mode)
{
	if(hDevice<0) return FALSE;
	if(mode>4) mode=4;
	if(mode<0) mode=0;
	if(lastMode==mode) return TRUE;
	int	bIsRunning=IsRunning;
	if(bIsRunning) CodecStop();

	myPara.mode=mode;
	switch(myPara.mode)
		{
		case	0:	 IQ_freq=50000;		IQ_spow=11;	 	DDCType=3;	break; //SampleRate=     50000,Bandwidth=    40000,BitsPerSample=32
		case	1:	 IQ_freq=100000;	IQ_spow=12;		DDCType=6;	break; //SampleRate=    100000,Bandwidth=    80000,BitsPerSample=32
		default:	 myPara.mode=2;
		case	2:	 IQ_freq=200000;	IQ_spow=13;		DDCType=9;	break; //SampleRate=    200000,Bandwidth=   160000,BitsPerSample=32
		case	3:	 IQ_freq=500000;	IQ_spow=14;		DDCType=13;	break; //SampleRate=    500000,Bandwidth=   400000,BitsPerSample=32
		case	4:	 IQ_freq=1000000;	IQ_spow=14;		DDCType=16;	break; //SampleRate=   1000000,Bandwidth=   800000,BitsPerSample=32
		}

	IQ_spow=12;

	if(!DataBuf)
		{
		DataBuf		= new float[(1<<(IQ_spow+1))];
		}

	if(extIOCallback) 
		{
		extIOCallback(-1,100,0,0); // tells the ExtIQ-APP, that the sampling frequency was changed
		}

	if(bIsRunning) CodecStart();
	return TRUE;
}

int G33DDCDll::SetFrequency(int freq)
{
	if(freq<10) freq=10;
	if(freq>50000000) freq=50000000;
	lastFrq=freq;
	if(hDevice>=0) 
		{
		if(G33_SetPreselectors)
			{
			int Low,High;
			if(freq>=0		        && freq	<750000)	{Low=0;			High=10000000;}
			else if(freq>=750000	&& freq <1250000)	{Low=0;			High=20000000;}
			else if(freq>=12500000	&& freq <1750000)	{Low=1000000;	High=20000000;}
			else if(freq>=17500000	&& freq	<2250000)	{Low=1000000;	High=30000000;}
			else if(freq>=22500000	&& freq	<2750000)	{Low=2000000;	High=30000000;}
			else if(freq>=27500000	&& freq <3250000)	{Low=2000000;	High=40000000;}
			else if(freq>=32500000	&& freq <3750000)	{Low=3000000;	High=40000000;}
			else if(freq>=37500000	&& freq	<4250000)	{Low=3000000;	High=50000000;}
			else if(freq>=42500000	&& freq	<4750000)	{Low=4000000;	High=50000000;}
			else if(freq>=47500000	&& freq <5250000)	{Low=4000000;	High=65000000;}
			else if(freq>=52500000	&& freq <6250000)	{Low=5000000;	High=65000000;}
			else if(freq>=62500000  && freq	<6750000)	{Low=5000000;	High=80000000;}
			else if(freq>=67500000	&& freq	<7750000)	{Low=6500000;	High=80000000;}
			else if(freq>=77500000	&& freq <8250000)	{Low=6500000;	High=95000000;}
			else if(freq>=82500000	&& freq <9250000)	{Low=8000000;	High=95000000;}
			else if(freq>=92500000	&& freq	<9750000)	{Low=8000000;	High=11000000;}
			else if(freq>=97500000	&& freq	<10750000)	{Low=9500000;	High=11000000;}
			else if(freq>=107500000	&& freq >11250000)  {Low=9500000;	High=13000000;}
			else if(freq>=112500000	&& freq <12750000)	{Low=11000000;	High=13000000;}
			else if(freq>=127500000	&& freq	<13250000)	{Low=11000000;	High=15000000;}
			else if(freq>=132500000	&& freq	<14750000)	{Low=13000000;	High=15000000;}
			else if(freq>=147500000	&& freq <15250000)	{Low=13000000;	High=17000000;}
			else if(freq>=152500000	&& freq <16750000)	{Low=15000000;	High=17000000;}
			else if(freq>=167500000	&& freq	<17250000)	{Low=15000000;	High=20000000;}
			else if(freq>=172500000	&& freq	<19750000)	{Low=17000000;	High=20000000;}
			else if(freq>=197500000	&& freq <20250000)	{Low=17000000;	High=25000000;}
			else if(freq>=202500000	&& freq <24750000)	{Low=20000000;	High=25000000;}
			else if(freq>=247500000	&& freq	<26000000)	{Low=20000000;	High=50000000;}
			else if(freq>=260000000	&& freq <50000000)	{Low=25000000;	High=50000000;}

			if(lastLow!=Low || lastHigh!=High)
				{
				lastLow=Low;
				lastHigh=High;
				if(G33_SetPreselectors) G33_SetPreselectors(hDevice,Low,High);
				}
			}
		return G33_SetFrequency(hDevice,0,freq);	
		}
	return freq;
}

void G33DDCDll::SetPower(BOOL onoff)
{
	if (hDevice>=0)
		{
		int status=0;
		G33_GetPower(hDevice,&status);
		if(onoff)
			{
			if(!status) G33_SetPower(hDevice,1);
			status=1; 
			}
		else
			{
			if(status) G33_SetPower(hDevice,0);
			status=3;
			}
		G33_SetLED(hDevice,status); 
		}
}

void G33DDCDll::_SetATT(int att)
{
	//Attenuator value in dB. Permitted values are: 0,3,6,9,12,15,18,21. 
	if (hDevice>=0)
		{
		myPara.ATT=att;
		if(att)	G33_SetAttenuator(hDevice,21);
		else	G33_SetAttenuator(hDevice,0);
		}
}

void G33DDCDll::_SetDither(BOOL enable)
{
	if (hDevice>=0)	
		{
		myPara.DIT=enable;
		G33_SetDithering(hDevice,enable);
		}
}

void G33DDCDll::_SetNB(BOOL enable)
{
	if (hDevice>=0)	
		{
		myPara.NB=enable;
		G33_SetADCNoiseBlanker(hDevice,0,enable);
		}
}

void G33DDCDll::_SetPRE(BOOL pre)
{
	if (hDevice>=0)	
		{
		myPara.PRE=pre;
		G33_SetPreamp(hDevice,pre);
		}
}
