////////////////////////////////////////////////////
////////////////////////////////////////////////////
// !!! this is not public domain !!!
// You can do what you want if it never hurt bonito.
// Otherwise, you will need the consent of Bonito.	
////////////////////////////////////////////////////
// RadioJet Open Source Project ExtIO_RJ10
// www.bonito.net/radiojet/opensource
////////////////////////////////////////////////////
////////////////////////////////////////////////////
#include "stdafx.h"
#include "GlobalTools.h"

#include "LC_ExtIO_Types.h"

#include <string.h>
#include <stdio.h>
#include <math.h>


pfnExtIOCallback	pfnCallback = 0;

#define SETTINGS_IDENTIFIER		"RadioJet 1102S 1.x"
int extIoAppSupportsSettings = 0;		// assume not supported
int extIoSettingsValid = 0;				// assume not valid
int extIoSetATT = -1;		// "Attenuator"
int extIoSetPRE = -1;		// "PreAmplifier"
int extIoSetWinTop = -1;	// "WindowTop"
int extIoSetWinLeft = -1;	// "WindowLeft"


BOOL WINAPI InitHW(char *name, char *model, int& type)
{
	type = 4;	//data returned via the sound card
				//this DLL controls only the frequency 
	strcpy(name,"RadioJet");
	strcpy(model,"1102S");
	if(!myCtrl.GetSafeHwnd())
		{
		AFX_MANAGE_STATE(AfxGetStaticModuleState( )); 
		myCtrl.Create(IDD_EXTIO_DIALOG,NULL);
		if(!myCtrl.ReadReady) 
			{
			AfxMessageBox("Radiojet not ready");
			return FALSE;
			}
		}

	return TRUE;
}

BOOL WINAPI OpenHW(void)
{
	if ( pfnCallback )
	{
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_Changed_ATT );
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_Changed_MGC );
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_RX_ChanMode_RIGHT );
	}
	return TRUE;
}

int WINAPI StartHW(long freq)
{
	myCtrl.Set_Frequency(freq);
	return 0;
}

void WINAPI StopHW(void)
{
}

void WINAPI CloseHW(void)
{
	myCtrl.Show_GUI(SW_HIDE);
}

int WINAPI SetHWLO(long LOfreq)
{
	myCtrl.Set_Frequency(LOfreq);
	return lastFrq;
}

long WINAPI GetHWLO(void)
{
	return lastFrq;
}

/* optional:
long WINAPI GetHWSR(void)
{
	return NULL;
}
*/

/* optional:
long WINAPI GetTune(void)
{
	return lastFrq;
}
*/

/* optional:
void WINAPI GetFilters(int& loCut, int& hiCut, int& pitch)
{
}
*/

/* optional:
char WINAPI GetMode(void)
{
	return NULL;
}
*/

/* optional:
void WINAPI ModeChanged(char mode)
{
}
*/

//this prevent the WARNING LNK4022 (cannot find unique match for symbol "GetStatus")
//too many GetStatus-Routines are defined in the system 
extern "C"
int __stdcall GetStatus(void)
{
return 0; // status not supported by this specific HW,
}


void WINAPI ShowGUI(void)
{
	myCtrl.Show_GUI(SW_SHOW);
}

void WINAPI HideGUI(void)
{
	myCtrl.Show_GUI(SW_HIDE);
}

void WINAPI SwitchGUI(void)
{
	myCtrl.Switch_GUI();
}


/* optional:
void WINAPI IFLimitsChanged(long low, long high)
{
}
*/

/* optional:
void WINAPI TuneChanged(long freq)
{
}
*/

void WINAPI SetCallback( pfnExtIOCallback funcptr )
{
	pfnCallback = funcptr;
}

/* optional:
void WINAPI RawDataReady(long samprate, int *Ldata, int *Rdata, int numsamples)
{
}
*/


long WINAPI ExtIoGetBandwidth(int srate_idx)
{
	// returns <= 0 on error
	if ( srate_idx == 0 )
		return 15000;  // 
	return 0;
}

long WINAPI ExtIoGetBwCenter(int srate_idx)
{
	// returns 0 on error, which is default
	if ( srate_idx == 0 )
		return 12000;
	return 0;
}


int WINAPI GetAttenuators( int atten_idx, float * attenuation )
{
	switch ( atten_idx )
	{
	case 0:		*attenuation = -15.0F;			return 0;
	case 1:		*attenuation =   0.0F;			return 0;
	default:	return 1;
	}
	return 1;
}

int WINAPI GetActualAttIdx(void)
{
	if ( myCtrl.isATT() )
		return 0;
	else
		return 1;
}

int WINAPI SetAttenuator( int atten_idx )
{
	myCtrl.Set_ATT( 0 == atten_idx );
	if ( GetActualAttIdx() == atten_idx )
		return 0;
	else
		return 1;	// ERROR
}


//---------------------------------------------------------------------------

int WINAPI ExtIoGetMGCs(int mgc_idx, float * gain)
{
  // fill in gain
	switch ( mgc_idx )
	{
	case 0:		*gain = -27.0F;			return 0;
	case 1:		*gain =  27.0F;			return 0;
	default:	return 1;
	}
	return 1;
}

int WINAPI ExtIoGetActualMgcIdx(void)
{
	if ( myCtrl.isDX() )
		return 1;
	else
		return 0;
}

int WINAPI ExtIoSetMGC(int mgc_idx)
{
  // set MGC / IFgain
	myCtrl.Set_PRE( 1 == mgc_idx );
	if ( ExtIoGetActualMgcIdx() == mgc_idx )
		return 0;
	else
		return 1;	// ERROR
}

//---------------------------------------------------------------------------

int  WINAPI ExtIoGetSetting( int idx, char * description, char * value )
{
	myCtrl.prepareSaveSettings();

	switch ( idx )
	{
	case 0: _snprintf( description, 1024, "%s", "Identifier" );		_snprintf( value, 1024, "%s", SETTINGS_IDENTIFIER );	return 0;
	case 1:	_snprintf( description, 1024, "%s", "Attenuator" );		_snprintf( value, 1024, "%d", extIoSetATT );			return 0;
	case 2:	_snprintf( description, 1024, "%s", "PreAmplifier" );	_snprintf( value, 1024, "%d", extIoSetPRE );			return 0;
	case 3:	_snprintf( description, 1024, "%s", "WindowTop" );		_snprintf( value, 1024, "%d", extIoSetWinTop );			return 0;
	case 4:	_snprintf( description, 1024, "%s", "WindowLeft" );		_snprintf( value, 1024, "%d", extIoSetWinLeft );		return 0;
	default:	return -1;	// ERROR
	}
	return -1;	// ERROR
}


void WINAPI ExtIoSetSetting( int idx, const char * value )
{
	int tempInt;
	// now we know that there's no need to save our settings into some (.ini) file,
	// what won't be possible without admin rights!!!,
	// if the program (and ExtIO) is installed in C:\Program files\..
	extIoAppSupportsSettings = 1;
	if ( idx != 0 && !extIoSettingsValid )
		return;	// ignore settings for some other ExtIO

	switch ( idx )
	{
	case 0:		extIoSettingsValid = ( value && !strcmp( value, SETTINGS_IDENTIFIER ) );
				// make identifier version specific??? - or not ==> never change order of idx!
				break;
	case 1:		tempInt = atoi( value );	extIoSetATT = tempInt;		break;
	case 2:		tempInt = atoi( value );	extIoSetPRE = tempInt;		break;
	case 3:		tempInt = atoi( value );	extIoSetWinTop = tempInt;	break;
	case 4:		tempInt = atoi( value );	extIoSetWinLeft = tempInt;	break;
	default:	break;
	}
}

//---------------------------------------------------------------------------



//------------------------------------------
// expansions of dll routines and funktions
//------------------------------------------

void WINAPI InitExtIOPara(void* userPara)
{
	memcpy(&myPara,userPara,sizeof(EXTIQ_PARA));
}

HWND WINAPI GetExPara(void* userPara)
{
	memcpy(userPara,&myPara,sizeof(EXTIQ_PARA));
	return myCtrl.m_hWnd;
}

void WINAPI SetExPara(EXTIQ_PARA *userPara)
{
	memcpy(&myPara,userPara,sizeof(EXTIQ_PARA));
	myCtrl.Set_ATT(myPara.ATT);
	myCtrl.Set_PRE(myPara.PRE);
}

void WINAPI SetAtt(int onoff)
{
	myCtrl.Set_ATT(onoff);
}

void WINAPI SetRxDxChn(int rxdx)
{
	myCtrl.Set_PRE(rxdx);
}



