// ExternDlg.cpp : implementation file
//
////////////////////////////////////////////////////
////////////////////////////////////////////////////
// !!! this is not public domain !!!
// You can do what you want if it never hurt bonito.
// Otherwise, you will need the consent of Bonito.	
////////////////////////////////////////////////////
// RadioJet Open Source Project ExtIO_RJ10
// www.bonito.net/radiojet/opensource
////////////////////////////////////////////////////
////////////////////////////////////////////////////

#include "stdafx.h"
#include "GlobalTools.h"

#include "LC_ExtIO_Types.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern pfnExtIOCallback	pfnCallback;
extern int extIoAppSupportsSettings;		// assume not supported
extern int extIoSettingsValid;				// assume not valid
extern int extIoSetATT;		// "Attenuator"
extern int extIoSetPRE;		// "PreAmplifier"
extern int extIoSetWinTop;	// "WindowTop"
extern int extIoSetWinLeft;	// "WindowLeft"


/////////////////////////////////////////////////////////////////////////////
// CExternDlg dialog

CExternDlg::CExternDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExternDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExternDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	visibility=SWP_HIDEWINDOW;
}

CExternDlg::~CExternDlg()
{
	if (m_hWnd != NULL)
		{
		DestroyWindow();
		}
}

void CExternDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExternDlg)
	DDX_Control(pDX, IDC_ATT,		b_ATT);
	DDX_Control(pDX, IDC_CHN_RX,	b_RX);
	DDX_Control(pDX, IDC_CHN_DX,	b_DX);
	DDX_Control(pDX, IDC_MANUAL,	b_manual);
	DDX_Control(pDX, IDC_LOGO,		b_logo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CExternDlg, CDialog)
	//{{AFX_MSG_MAP(CExternDlg)
	ON_BN_CLICKED(IDC_ATT,		OnAtt)
	ON_BN_CLICKED(IDC_CHN_DX,	OnChnDx)
	ON_BN_CLICKED(IDC_CHN_RX,	OnChnRx)
	ON_BN_CLICKED(IDC_MANUAL,	OnManual)
	ON_BN_CLICKED(IDC_LOGO,		OnManual)
	ON_WM_CLOSE()
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExternDlg message handlers

BOOL CExternDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	DWORD  dwGetVer = GetVersion();
	int WinVersion=LOBYTE(LOWORD(dwGetVer));

	if(WinVersion>5) 
		{
		 //window 7 have an light background
		m_hIcon = AfxGetApp()->LoadIcon(IDR_RJ10_ICON); 
		}
	else
		{
		//XP have an dark background
		m_hIcon = AfxGetApp()->LoadIcon(IDR_RJ10_ICON_XP);	
		}

	SetIcon(m_hIcon, FALSE);		// Set small icon
	b_logo.m_bButtonStyle=1;
	b_logo.SetImage(IDB_RJ10LOGO);

	// if myPara.msgHWND != NULL, then this ExtIO.dll is controlled by an application other than Winrad or clone.
	// Then, the communication with the Mircrosoft Message System is implemented - not the Mircrosoft untypical callback manner

	if ( extIoAppSupportsSettings && extIoSettingsValid )
	{
		myPara.ATT=extIoSetATT;		// "Attenuator"
		myPara.ATT&=3;
		myPara.PRE=extIoSetPRE;		// "PreAmplifier"
		myPara.PRE&=1;
	}
	else if(!myPara.msgHWND)
	{
		myPara.ATT=GetProfileInt("PARAMETER","Attenuator",0);
		myPara.ATT&=3;
		myPara.PRE=GetProfileInt("PARAMETER","PreAmplifier",0);//we use this here as RXDX shwitch 
		myPara.PRE&=1;
	}
	//else the parameter are all ready set of InitExtIOPara

	b_manual.SetBold(1); 
	b_manual.SetColor(RGB(196,0,0),1);
	b_manual.SetStyle(1);

	Open();
	b_ATT.SetCheck(myPara.ATT);
	b_DX.SetCheck(myPara.PRE);
	b_RX.SetCheck(!myPara.PRE);

	CRect re;
	GetWindowRect(re);
	if ( extIoAppSupportsSettings && extIoSettingsValid )
	{
		re.top = extIoSetWinTop;	// "WindowTop"
		re.left = extIoSetWinLeft;	// "WindowLeft"
	}
	else
	{
		re.top = GetProfileInt("POSITION","WindowTop",re.top);
		re.left = GetProfileInt("POSITION","WindowLeft",re.left);
	}
	SetWindowPos(NULL,re.left,re.top,0,0,SWP_NOACTIVATE|SWP_NOSIZE|visibility);

	return TRUE;
}

void CExternDlg::OnCancel() 
{
	ShowWindow(SW_HIDE);
	if(myPara.msgHWND) ::PostMessage(myPara.msgHWND,myPara.msgSHOW,SW_HIDE,SW_HIDE);
}
void CExternDlg::OnOK() {}
void CExternDlg::OnClose() 
{
	ShowWindow(SW_HIDE);
	if(myPara.msgHWND) ::PostMessage(myPara.msgHWND,myPara.msgSHOW,SW_HIDE,SW_HIDE);
}

void CExternDlg::OnManual()
{
	ShellExecute(NULL, NULL,"http://www.bonito.net/manuals/radiojet/opensource", NULL,NULL, SW_SHOWNORMAL);
}

void CExternDlg::Set_ATT(int set)
{
	_SetATT(set);
	if(myPara.msgHWND && myPara.msgATT) ::PostMessage(myPara.msgHWND,myPara.msgATT,myPara.ATT,(LPARAM)m_hWnd);
	b_ATT.SetCheck(myPara.ATT);
}

void CExternDlg::OnAtt()
{
	if(mR_ATT) Set_ATT(0);
	else Set_ATT(1);

	if ( pfnCallback )
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_Changed_ATT );
}

void CExternDlg::Set_PRE(int set)
{
	_SetPRE(set);
	if(myPara.msgHWND && myPara.msgPRE) ::PostMessage(myPara.msgHWND,myPara.msgPRE,myPara.PRE,(LPARAM)m_hWnd);
	b_DX.SetCheck(myPara.PRE);
	b_RX.SetCheck(!myPara.PRE);
}

void CExternDlg::OnChnDx()
{
	Set_PRE(1);
	if ( pfnCallback )
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_Changed_MGC );
}

void CExternDlg::OnChnRx()
{
	Set_PRE(0);
	if ( pfnCallback )
		EXTIO_STATUS_CHANGE( pfnCallback, extHw_Changed_MGC );
}

void CExternDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);
	myPara.SHOW=bShow; //set the status
	if(myPara.msgHWND) ::PostMessage(myPara.msgHWND,myPara.msgSHOW,bShow,nStatus);
}

void CExternDlg::Show_GUI(int show)
{
	if(show) visibility=SWP_SHOWWINDOW;
	else visibility=SWP_HIDEWINDOW;
	if(myCtrl.GetSafeHwnd())
		{
		if(show)
			{
			if(!IsWindowVisible()) ShowWindow(SW_SHOW);
			SetForegroundWindow();
			}
		else
			{
			ShowWindow(SW_HIDE);
			}
		}
}

void CExternDlg::Switch_GUI()
{
	Show_GUI( !IsWindowVisible() );
}


void CExternDlg::prepareSaveSettings()
{
	static int prepCalled = 0;
	if ( 0 == prepCalled )
	{
		CRect re;
		GetWindowRect(re);

		extIoSetATT		= myPara.ATT;	// "Attenuator"
		extIoSetPRE		= myPara.PRE;	// "PreAmplifier"
		extIoSetWinTop	= re.top;		// "WindowTop"
		extIoSetWinLeft	= re.left;		// "WindowLeft"
		prepCalled = 1;
	}
}

void CExternDlg::OnDestroy() 
{
	CloseDLL();

	prepareSaveSettings();
	if ( !extIoAppSupportsSettings )
	{
		if(!myPara.msgHWND)
			{
			WriteProfileInt("PARAMETER","Attenuator",myPara.ATT);
			WriteProfileInt("PARAMETER","PreAmplifier",myPara.PRE);  // here RXDX switch state
			}

		CRect re;
		GetWindowRect(re);
		WriteProfileInt("POSITION","WindowTop",re.top);
		WriteProfileInt("POSITION","WindowLeft",re.left);
	}

	CDialog::OnDestroy();
}

