////////////////////////////////////////////////////
////////////////////////////////////////////////////
// !!! this is not public domain !!!
// You can do what you want if it never hurt bonito.
// Otherwise, you will need the consent of Bonito.	
////////////////////////////////////////////////////
// RadioJet Open Source Project ExtIO_RJ10
// www.bonito.net/radiojet/opensource
////////////////////////////////////////////////////
////////////////////////////////////////////////////
#include "stdafx.h"
#include "Si570Driver.h"
#include "GlobalTools.h"

CSi570Driver::CSi570Driver()
{
	DDSFrq=lastFrq=177000; 
	last_hsdiv=last_n1=0;
    currentRfreq=currentFreq=1;
	mR_WRT=0;
	RJ10_OUT_START_UP=15.0;
	corfacDiv=400000.;
}

CSi570Driver::~CSi570Driver()	
{
}

BOOL CSi570Driver::Open(UINT PID)
{	
	ImOpen=OpenDevice(PID);
	if(!ImOpen) return ImOpen;

	switch(mCalib.StartUpFrq)
		{
		case	1:	RJ10_OUT_START_UP=15.0; break;
		default:
		case	2:	RJ10_OUT_START_UP=56.32; break;
		}

	Reset();

	if(!ReadReady) return -1;

	return TRUE;
}

void CSi570Driver::CheckStartUpFrq()
{
	double	startUP=56.32;

	switch(mCalib.StartUpFrq)
		{
		case	1:	startUP=15.0; break;
		default:
		case	2:	startUP=56.32; break;
		}

	if(RJ10_OUT_START_UP!=startUP) 
		{
		RJ10_OUT_START_UP=startUP;
		si570reset();
		SetFreq(lastFrq);
		}
}

void CSi570Driver::_SetATT(int att)
{
	if(att) mR_ATT=ATT_BIT;
	else mR_ATT=0;
	SetGPIOLevel();
	myPara.ATT=att&1;
}

void CSi570Driver::_SetPRE(int pre) //set preamplifire
{
	if(pre) mR_PRE=PRE_BIT; 
	else mR_PRE=0;
	SetGPIOLevel();
	myPara.PRE=pre&1;
}

void CSi570Driver::Reset()
{
	if(ImOpen) 
		{
		si570reset();
		Refresh_Frequency();
		}
}

void CSi570Driver::si570reset(void) 
{
	UINT	frac_bits;
	UBYTE	buf[8];


    buf[0] = 0x01;   

	EnterCriticalSection(&m_CriticalSection);   
	SetI2CDeviceAddress(0xAA); 
	WaitAction=0xAA; //never set if this is in action
	rmWriteDataIntoI2C(135,buf,1);//send reset
	rmReadDataFromI2C(0x07,buf,6);//read the byte 7-12 
	WaitAction=0;
	LeaveCriticalSection(&m_CriticalSection);

     // divider conversion
    hsdiv = ((buf[0] & 0xE0) >> 5) + 4; 
    n1 = (( buf[0] & 0x1F ) << 2 ) + (( buf[1] & 0xC0 ) >> 6 );  
    if (n1 == 0) n1 = 1;
	else if ((n1 & 1) != 0) n1 = n1++;

    frac_bits = (( buf[2] & 0xF ) * RJ10_POW_2_24 );
    frac_bits = frac_bits + (buf[3] * RJ10_POW_2_16);
    frac_bits = frac_bits + (buf[4] * 256);
    frac_bits = frac_bits + buf[5];

    rfreq = frac_bits;
    rfreq = rfreq / RJ10_POW_2_28;
    rfreq = rfreq + ( (( buf[1] & 0x3F ) << 4 ) + (( buf[2] & 0xF0 ) >> 4 ) );

    fxtal_device=fxtal=(RJ10_OUT_START_UP * n1 * hsdiv) / rfreq; //in 
}

#define UN_FREEZE	0x00
#define FREEZE_DCO	0x10
#define FREEZE_M	0x20
#define NEW_FRQ		0x40

#define COMAND_1	wbuf[0]
#define COMAND_2	wbuf[1]
#define COMAND_3	wbuf[2]
#define COMAND_4	wbuf[3]

#define COMAND_FRZ_NEW_FRQ	wbuf[5]

#define W_REG7		writebuf[0]
#define W_REG8		writebuf[1]
#define W_REG9		writebuf[2]
#define W_REG10		writebuf[3]
#define W_REG11		writebuf[4]
#define W_REG12		writebuf[5]

#define xRJ10_FDCO_MIN		4850000000I64
#define xRJ10_FDCO_MAX		5670000000I64

volatile CWinThread*  FRQ_Thread=NULL; 
CSi570Driver *myIFjet;
UINT MyFrqThread(LPVOID)
{
	while(lastFrq!=myIFjet->DDSFrq) 
		{
		myIFjet->SetFreq(lastFrq);
		}
	FRQ_Thread=NULL;
	return 0;
}

void CSi570Driver::Refresh_Frequency()
{
	DDSFrq=0;
	last_n1=last_hsdiv=0;
	currentFreq=currentRfreq=1;
	SetFreq(lastFrq);
}


//set frq in a background thread, for fast frq.slide,- this will overwrite frequencies that were too slow
int CSi570Driver::Set_Frequency(int freq)
{
	if(freq<10) freq=10;
	if(freq>38000000) freq=38000000;
	if(DDSFrq==freq) return freq;
	lastFrq=freq;

	if(ImOpen)
		{
		myIFjet=this;
		if(!FRQ_Thread) 
			{
			FRQ_Thread= AfxBeginThread(MyFrqThread,NULL,THREAD_PRIORITY_LOWEST);
			}	
		}
	return lastFrq;
}

void CSi570Driver::SetFreq(int frq)
{
	UINT64	frequency;
	if(ImOpen)
		{
		DDSFrq=frq;
		if(frq)
			{
#ifdef 	SDR_LO_OFFSET
			frq+=SDR_LO_OFFSET;  //SDR Offset for LOFreq
#endif
			frequency=(INT64)((double)frq+((double)mCalib.ScaleCorrecture/30000000.*(double)frq))+45000000;
			INT64 z=frequency - currentFreq; if(z<0) z*=-1;		
			z=(z*10000)/currentFreq;
			if(z <35) 
				{
				Set_ShortFreq(frequency);
				}
			else
				{
				Set_LongFreq(frequency);
				}
			}
		}
}

void CSi570Driver::Set_ShortFreq(UINT64 frq)
{
    UINT	whole;
    UBYTE	wbuf[8]={0};
    UBYTE	writebuf[8]={0};
	UINT64	datarfreq;
	UBYTE	*frac_byte=(UBYTE*)&datarfreq;

    datarfreq = currentRfreq * frq / currentFreq;
	whole=(UINT)(datarfreq>>28);
    W_REG8 =((last_n1 & 3) << 6) | (whole>>4); 
	W_REG9=((whole&0x000F)<<4)|(frac_byte[3]&0x0F);
	W_REG10=frac_byte[2];		
	W_REG11=frac_byte[1];
	W_REG12=frac_byte[0];

	COMAND_1=FREEZE_M; 	
	COMAND_2=UN_FREEZE;  

	EnterCriticalSection(&m_CriticalSection);   
	SetI2CDeviceAddress(0xAA); 
	WaitAction=0xAA; //never set if this is in action
	rmWriteDataIntoI2C(135, &COMAND_1,1);	//set the Freeze M bit in that register
	rmWriteDataIntoI2C(8,&W_REG8,5);		//set register 8-12
	rmWriteDataIntoI2C(135, &COMAND_2,1);	//clear the M bit in that register
	WaitAction=0;
	LeaveCriticalSection(&m_CriticalSection);
}

void CSi570Driver::Set_LongFreq(UINT64 frq)
{
    UINT	whole;
	UBYTE	HS_DIV[6] = {11, 9, 7, 6, 5, 4};
    UBYTE	writebuf[8]={0};
    UBYTE	wbuf[8]={0};

	UINT64	best_rfreq,datafxtal;
	UBYTE	best_hsdiv,best_n1;
	INT64	fdco, best_fdco = xRJ10_FDCO_MAX;
	int		i;

	UBYTE	*frac_byte=(UBYTE*)&best_rfreq;

	hsdiv=n1=0;

	datafxtal=(UINT64)((fxtal+(mCalib.FreqCorrecture/corfacDiv))*1000000);

	for (i = 0; i < 6; i++) 
		{
		hsdiv = HS_DIV[i];
		/* Calculate lowest possible value for n1 */
		n1 = (UBYTE) (xRJ10_FDCO_MIN / hsdiv / frq);
		if (!n1 || (n1 & 1)) n1++;
		while (n1 <= 128) 
			{
			fdco = frq * hsdiv * n1;
			if (fdco > xRJ10_FDCO_MAX) break;
			if (fdco > xRJ10_FDCO_MIN && fdco < best_fdco) 
				{
				best_n1 = n1;
				best_hsdiv = hsdiv;
				best_rfreq = (fdco << 28I64) / datafxtal;
				best_fdco = fdco;
				}
			n1 += (n1 == 1 ? 1 : 2);
			}
		}
	if(best_fdco==xRJ10_FDCO_MAX) return;

	n1=best_n1;
	hsdiv=best_hsdiv;
	whole=(UINT)(best_rfreq>>28I64);
	currentRfreq=best_rfreq;
	currentFreq=frq;

	fxtal_device=fxtal+(mCalib.FreqCorrecture/corfacDiv);
    rfreq=(((UINT)frq/1000000.) * n1 * hsdiv)/fxtal_device; 

	hsdiv-=4;  n1-=1;
	last_n1=n1; last_hsdiv=hsdiv;

	//set this all to the register bits
    W_REG7 = (hsdiv << 5) | (n1 >> 2);
    W_REG8 =((n1 & 3) << 6) | (whole>>4); 
	W_REG9=((whole&0x000F)<<4)|(frac_byte[3]&0x0F);
	W_REG10=frac_byte[2];		
	W_REG11=frac_byte[1];
	W_REG12=frac_byte[0];

	COMAND_1=FREEZE_DCO; 	
	COMAND_2=UN_FREEZE;  
	COMAND_3=NEW_FRQ;
	COMAND_4=FREEZE_M;
	wbuf[5]=UN_FREEZE;
	wbuf[6]=0;
	wbuf[7]=NEW_FRQ;

	EnterCriticalSection(&m_CriticalSection);   
	SetI2CDeviceAddress(0xAA); 
	WaitAction=0xAA; //never set if this is in action
	rmWriteDataIntoI2C(135, &COMAND_4,1);	//set the Freeze M bit in that register
///	rmWriteDataIntoI2C(137, &COMAND_1,1);	//freeze the DCO 
	rmWriteDataIntoI2C(7,writebuf,6);		//set register 7-12
//	rmWriteDataIntoI2C(137, &COMAND_2,1);	//UnFreeze the DCO (bit 4 of Register 137)
	rmWriteDataIntoI2C(135, &COMAND_3,1);	//Unfreeze the DCO and assert the NewFreq bit (bit 6 of Register 135)
	WaitAction=0;
	LeaveCriticalSection(&m_CriticalSection); 

}

