//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SDR14Ctl.h"
#include "SDR14params.h"
#include "../ExtIOFunktions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//sample rate that above which the USB cannot keep up and so the data
// cannot be contiguous.
#define MAX_CONTIGUOUS_FREQ 197000

#define MAX_AUX_BYTES 24

#define DATA_BLOCK_SIZE 4096	//block size in 16 bit words of the SDR-14 data.
#define DATA_IQ_PAIRS 2048		//the number of IQ pairs of the SDR-14 data.

//#define BLOCK_FIFO_SIZE 32	//must be power of 2
#define BLOCK_FIFO_SIZE 8	//must be power of 2

#define CAPTURE_MODE_CONTIGUOUS 0	//Three capture modes of the SDR-14
#define CAPTURE_MODE_CONTINUOUS 1
#define CAPTURE_MODE_ONESHOT 2

#define DATA_ACK_TIMEOUT_VALUE ((66666666*1)/2048)	//1 seconds of 2048 sample blocks

#define IQ_OFFSET_CONST 307.0	//Used for NCO spur removal

#define MAX_WINMSG_LIMIT 1

#define IOEEROM_SAMPLERATE 0	//External IO EEROM memory address definitions for sdriq

#define NOMINAL_ADFREQ 66666667


//////////////////////////////////////////////////////////////////////////////
// sdr14ctrl::sdr14ctrl - Constructor

sdr14ctrl::sdr14ctrl()
{
	ClearVariables();
	InitializeCriticalSection(&m_CriticalSection);
}

/////////////////////////////////////////////////////////////////////////////
// sdr14ctrl::~sdr14ctrl - Destructor

sdr14ctrl::~sdr14ctrl()
{
	Exit();
	DeleteCriticalSection(&m_CriticalSection);
}

void sdr14ctrl::Exit()
{
	Stop();
	EndThreads();
	if(m_pDataBuffer)
		{
		for(int i=0; i<BLOCK_FIFO_SIZE; i++)
			{
			if(m_pDataBuffer[i])
				{
				delete[] m_pDataBuffer[i];
				m_pDataBuffer[i] = NULL;
				}
			}
		delete[] m_pDataBuffer;
		m_pDataBuffer = NULL;
		}
	DeleteResources();
	ClearVariables();
}

void sdr14ctrl::ClearVariables()
{
	m_ThreadRunning = NULL;
	m_pUSBThread = NULL;
	m_pFIFOThread = NULL;
	m_pDataBuffer = NULL;
	m_DeviceName = _T("Unknown");
	m_SerialNumber= _T("000000");
	m_BootRevision = 0;
	m_FirmwareRevision = 0;
	m_HFgain = 0;
	m_IFgain = 0;
	m_IFfilter = 0;
	m_InputSource = 0;
	m_InterfaceVersion = 0;
	m_NCOfrequency = 153000;
	m_ClockFrequency = 66666667;
	m_CaptureMode = CAPTURE_MODE_CONTIGUOUS;
	m_SampleRate = m_ClockFrequency;
	m_NumberOfBlocks = -1;
	m_Complex = FALSE;
	m_IQOffset = 0.0;
	m_FifoHead = 0;
	m_FifoTail = 0;
	m_FifoFFTTail = 0;
	m_TestModeChannel = 0;
	m_ModifiedFlag=FALSE;
	m_LastStatus=0;
	dBcorr=1.f;
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//   L o c a l    H e l p e r    F u n c t i o n s
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main USB process loop
//////////////////////////////////////////////////////////////////////
UINT sdr14ctrl::USBThreadProc(LPVOID pParam)
{
	sdr14ctrl* pSDR14Ctrl = (sdr14ctrl*)pParam;
	Sleep(500);		//don't start right away
	pSDR14Ctrl->sdr14USBio::USBthreadProc( pSDR14Ctrl);
	return 2;
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main FIFO process loop
//////////////////////////////////////////////////////////////////////
UINT sdr14ctrl::FIFOThreadProc(LPVOID pParam)
{
	sdr14ctrl* pSDR14Ctrl = (sdr14ctrl*)pParam;
	Sleep(300);		//don't start right away
	pSDR14Ctrl->sdr14USBio::FIFOThreadProc( pSDR14Ctrl);
	return 3;
}

//////////////////////////////////////////////////////////////////////
//  Ends thread by telling it to stop and waiting on it to quit
//////////////////////////////////////////////////////////////////////

void sdr14ctrl::EndThreads()
{
	if(m_pUSBThread)
		{
		m_pUSBThread->SuspendThread();
		delete m_pUSBThread;
		m_pUSBThread = NULL;
		}
	if(m_pFIFOThread)
		{
		m_pFIFOThread->SuspendThread();
		delete m_pFIFOThread;
		m_pFIFOThread = NULL;
		}
	m_ThreadRunning = NULL;
}

///////////////////////////////////////////////////////////////////////////
//  Parses received ASCP format message from SDR-14
//  Called by worker thread in sdr14USBio object.
///////////////////////////////////////////////////////////////////////////
void sdr14ctrl::ParseASCPmsg(ASCPstruct* pASCPmsg)
{
	int i;


	switch( pASCPmsg->bytes.b1 & TYPE_MASK)	//ASCP message type
	{
		case TYPE_TARG_RESP_CITEM:
			switch( pASCPmsg->words.CItem )
			{
				case CI_GENERAL_INTERFACE_NAME:
					m_DeviceName = (char*)pASCPmsg->bytes.parms;
					ReadSDRIQClockFrequency();	//get calibrated reference frequency from USB chip EEROM
					WriteSDRIQClockFrequency();	//echo it back to the SDRIQ processor
					break;
				case CI_GENERAL_INTERFACE_SERIALNUM:
					m_SerialNumber = (char*)pASCPmsg->bytes.parms;
					break;
				case CI_GENERAL_HARDFIRM_VERSION:
					if(1 == pASCPmsg->bytes.parms[0])
						m_FirmwareRevision = (unsigned __int16)pASCPmsg->bytes.parms[1];
					else
						m_BootRevision = (unsigned __int16)pASCPmsg->bytes.parms[1];
					break;
				case CI_GENERAL_INTERFACE_VERSION:
					m_InterfaceVersion = pASCPmsg->words.parms[0];
					//signal status that the SDR-14 is ready for use
						::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					break;
				case CI_RX_STATE:
					if(pASCPmsg->bytes.parms[1] == RX_STATE_ON)
					{
						sdr14USBio::m_Running = TRUE;
						::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_RUNNING, 0);
					}
					else
					{
						sdr14USBio::m_Running = FALSE;
						::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					}
					break;
				case CI_RX_IF_GAIN:
					i=0;
					break;
				default:
TRACE("y");
					break;
			}
			break;
		case TYPE_TARG_UNSOLICITED_CITEM:
			switch( pASCPmsg->words.CItem )
			{
				case CI_RX_STATE:	//unsolicited on/offs used for data syncing
					if(pASCPmsg->bytes.parms[1] == RX_STATE_ON)
					{
						m_BlockCount = 0;
					}
					else if(pASCPmsg->bytes.parms[1] == RX_STATE_IDLE)
					{
						m_BlockCount = 0;
						sdr14USBio::m_Running = FALSE;
						::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					}
					else
					{
						m_BlockCount = 0;
						sdr14USBio::m_Running = FALSE;
						::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_ERROR, 0);
					}
					break;
				case CI_GENERAL_STATUS_CODE:	//unsolicited status msgs from sdr
					switch( pASCPmsg->bytes.parms[0] )
					{
						case GENERAL_STATUS_IDLE:
							if(!sdr14USBio::m_Running)
								::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
							break;
						case GENERAL_STATUS_BUSY:
							if(sdr14USBio::m_Running)
								::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_RUNNING, 0);
							break;
						case GENERAL_STATUS_6620LOADING:
							break;
						case GENERAL_STATUS_BOOTIDLE:
							break;
						case GENERAL_STATUS_BOOTBUSY:
							break;
						case GENERAL_STATUS_BOOTERROR:
							break;
						case GENERAL_STATUS_ADOVERLOAD:
							if(!m_ADOverload)
							{
							TRACE("\n\rOvr");
								::PostMessage( m_MSGhWnd, MSG_STATUSCHNG, STATUS_ADOVERLOAD, 0);
								m_ADOverload = TRUE;
							}
							break;
					}
					break;
				default:
TRACE("z");
					break;
			}
			break;
		case TYPE_TARG_RESP_CITEM_RANGE:
			break;
		case TYPE_DATA_ITEM_ACK:	//acks for all the data items that ack
			switch( pASCPmsg->words.CItem )
			{
				case 0:	//acks for data0 blocks (firmware update blocks)
					break;
				case 1: //acks for data1 blocks (6620 parameter loads)
					break;
				case 2: 
					break;
				case 3:
					break;
			}
			break;
		case TYPE_TARG_DATA_ITEM0:	//main data item from SDR-14
			EnterCriticalSection(&m_CriticalSection);
	
			//output data to the ExtIQ-Callback 
			if(m_Running)	
				{
				extIOCallback(DATA_IQ_PAIRS,0,0,(unsigned char*)pASCPmsg->data.parms);
				}

			LeaveCriticalSection(&m_CriticalSection);
			// send a Data0 ack every second or so to keep the SDR-14
			//  watchdog from timing out
			if(--m_DataAckCounter <=0)
			{
				m_DataAckCounter = m_DataAckCounterLoadVal;
				m_ASCPackmsg.words.header = DATA_ITEM_ACK_LENGTH;
				m_ASCPackmsg.bytes.b1 |= TYPE_DATA_ITEM_ACK;
				m_ASCPackmsg.bytes.b2 = 0;
				sdr14USBio::SendASCPMsg( &m_ASCPackmsg);
			}
			break;

		default:	TRACE("x");
			break;
	}
}

///////////////////////////////////////////////////////////////////////////
//  Creates/deletes data buffers.
///////////////////////////////////////////////////////////////////////////
void sdr14ctrl::SetupDataBuffers(int blocks)
{
	if(blocks>128)		//keep Block specifier bounded
		blocks = 128;
	if(blocks==0)
		blocks = 1;
	if( m_NumberOfBlocks != blocks )	//if blocks has changed
	{
		if(m_pDataBuffer)
		{
			for(int i=0; i<BLOCK_FIFO_SIZE; i++)
			{
				if(m_pDataBuffer[i])
				{
					delete[] m_pDataBuffer[i];
					m_pDataBuffer[i] = NULL;
				}
			}
			delete[] m_pDataBuffer;
		}
		m_NumberOfBlocks = blocks;
		// Allocate "main" array
		m_pDataBuffer = new short*[BLOCK_FIFO_SIZE];
		// Allocate "sub" arrays
		for(int i=0; i<BLOCK_FIFO_SIZE; i++)
			m_pDataBuffer[i] = new short[DATA_BLOCK_SIZE*m_NumberOfBlocks];
	}
	m_FifoHead = 0;
	m_FifoTail = 0;
	m_FifoFFTTail = 0;
	m_BlockCount = 0;

}

///////////////////////////////////////////////////////////
//  Calculates the AD6620 I/Q Offset that is needed to
// eliminate a spur located at the NCO frequency
// This spur is a function of IF gain and the number of
// FIR taps in the AD6620.
///////////////////////////////////////////////////////////
double sdr14ctrl::CalcOffsetIQ()
{
double div;
	switch(m_IFgain & 0x1F)
	{
		case 0:
			div = 16.0;
			break;
		case 6:
			div = 8.0;
			break;
		case 12:
			div = 4.0;
			break;
		case 18:
			div = 2.0;
			break;
		case 24:
			div = 1.0;
			break;
	}
	return ((double)(m_RCFTaps)/(div*IQ_OFFSET_CONST));
}


///////////////////////////////////////////////////////////
//  Since the SDR-IQ processor does not have any EEROM,
// the reference calibration frequency is stored in the
// FTDI USB chips user EEROM area.  This routine reads it.
///////////////////////////////////////////////////////////
void sdr14ctrl::ReadSDRIQClockFrequency()
{
	unsigned int val;

	if(SDRTYPE_IQ == m_SDRtype)
	{
		sdr14USBio::ReadIOMemory( IOEEROM_SAMPLERATE, &val);
		if( (val>50000000) && (val<70000000) )
		{
			m_ClockFrequency = val;
		}
		else	//write default to eerom
		{
			val = NOMINAL_ADFREQ;
			sdr14USBio::WriteIOMemory( IOEEROM_SAMPLERATE, val);
			m_ClockFrequency = val;
		}
	}
}

///////////////////////////////////////////////////////////
//  Since the SDR-IQ processor does not have any EEROM,
// the reference calibration frequency is stored in the
// FTDI USB chips user EEROM area.  This routine sends
// m_ClockFrequency back to the processor so it can be
// calibrated.
///////////////////////////////////////////////////////////
void sdr14ctrl::WriteSDRIQClockFrequency()
{
	if(SDRTYPE_IQ == m_SDRtype)
	{
		m_ASCPmsg.words.header = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_IN_SAMPLE_RATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
		m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_ClockFrequency&0xFF);
		m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_ClockFrequency>>8)&0xFF);
		m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_ClockFrequency>>16)&0xFF);
		m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_ClockFrequency>>24)&0xFF);
		sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	}
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
/////////// P r o p e r t y   A c c e s s    F u n c t i o n s //////
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

void sdr14ctrl::SetClockFrequency(long nNewValue) 
{
	if(SDRTYPE_IQ == m_SDRtype)
		sdr14USBio::WriteIOMemory( IOEEROM_SAMPLERATE, nNewValue);
	m_ClockFrequency = nNewValue;
	m_ASCPmsg.words.header = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_IN_SAMPLE_RATE;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_ClockFrequency&0xFF);
	m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_ClockFrequency>>8)&0xFF);
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_ClockFrequency>>16)&0xFF);
	m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_ClockFrequency>>24)&0xFF);
	sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

void sdr14ctrl::SetHFgain(short nNewValue) 
{
	nNewValue+=5;
	nNewValue/=10; nNewValue*=10;

	m_HFgain = nNewValue;
	m_ASCPmsg.words.header = CI_RX_RF_GAIN_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_RF_GAIN;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)0;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)m_HFgain;
	sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

void sdr14ctrl::SetIFfilter(short nNewValue) 
{
C6620Params	filter;
	if(m_IFfilter != nNewValue)	//can't load new filter while running
		Stop();
	if(SDRTYPE_IQ == m_SDRtype)
	{	//0 thru 5 and 12 are the only valid values for the IQ
		if( nNewValue > 5 )
			nNewValue = 12;		//force it to 190KHz if out of range of SDR-IQ
		m_Complex = TRUE;
	}
	m_IFfilter = nNewValue;
	//calculate new sample rate based on new IF filter
	if(m_Complex)
	{
		filter.LoadFixedFilter(m_IFfilter+1);
		m_SampleRate = m_ClockFrequency/filter.m_TotalDecimationRate;
		m_RCFTaps = filter.m_RCFTaps;
		m_IQOffset = CalcOffsetIQ();
	}
	else
	{
		m_SampleRate = m_ClockFrequency;
	}
	SetModifiedFlag();
}

void sdr14ctrl::SetInputSource(short nNewValue) 
{
	if( m_InputSource != nNewValue ) //must stop to change input src
		Stop();

	/*if(SDRTYPE_IQ == m_SDRtype)
	{
		nNewValue = 129;	//force SDR-IQ input to HF complex input source.
	}
	m_InputSource = nNewValue;
	*/


	m_InputSource = 0x81;	//here we force for RJ10-Paranorama  Ant-1 and  Complex Mode
	if( m_InputSource&(1<<7) )
	{
		m_Complex = TRUE;
	}
	else
	{
		m_Complex = FALSE;
		m_SampleRate = m_ClockFrequency;
	}
	m_IQOffset = CalcOffsetIQ();
	SetModifiedFlag();
}

void sdr14ctrl::SetNCOfrequency(long nNewValue) 
{
	m_NCOfrequency = nNewValue;
	m_ASCPmsg.words.header = CI_RX_FREQUENCY_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_FREQUENCY;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)0;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_NCOfrequency&0xFF);
	m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_NCOfrequency>>8)&0xFF);
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_NCOfrequency>>16)&0xFF);
	m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_NCOfrequency>>24)&0xFF);
	m_ASCPmsg.bytes.parms[5] = 1; //freq multiplier == x 1
	sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

void sdr14ctrl::SetIFgain(short nNewValue) 
{
	nNewValue+=3;
	nNewValue/=6; nNewValue*=6;

	m_IFgain = nNewValue;
	m_TestModeChannel = nNewValue & 0xE0;	//bits 7,6, and 5 are factory test mode bits
	m_IQOffset = CalcOffsetIQ();	//cacl a new IQ NCO spur removal offset
	m_ASCPmsg.words.header = CI_RX_IF_GAIN_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_IF_GAIN;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_IFgain & 0x1F);
	sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}


//////////////////////////////////////////////////////////////////////
//  Called to Initialize the Control and start the worker thread that
//  runs the USB port.
//////////////////////////////////////////////////////////////////////
void sdr14ctrl::InitControl() 
{
	if(m_ThreadRunning==0)	//no need to restart thread if running
		{
		EndThreads();	//if thread is already running stop it
		//Start up USB worker thread
		m_pUSBThread = AfxBeginThread(sdr14ctrl::USBThreadProc,
									this,
									THREAD_PRIORITY_TIME_CRITICAL,
									0,
									CREATE_SUSPENDED);
		m_pUSBThread->m_bAutoDelete = FALSE;
		m_ThreadRunning = 1;
		m_pUSBThread->ResumeThread();
		//Start up FIFO worker thread
		m_pFIFOThread = AfxBeginThread(sdr14ctrl::FIFOThreadProc,
									this,
									THREAD_PRIORITY_ABOVE_NORMAL,
									0,
									CREATE_SUSPENDED);
		m_pFIFOThread->m_bAutoDelete = FALSE;
		m_pFIFOThread->ResumeThread();
		}
}


//////////////////////////////////////////////////////////////////////
//  Method to send a raw data message to the AD6620
//////////////////////////////////////////////////////////////////////
void sdr14ctrl::Send6620RawCmd(short Address, short D0, short D1, short D2, short D3, short D4) 
{
	m_ASCPmsg.words.header = 2+7;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM1;
	m_ASCPmsg.dibytes.parms[0] = (unsigned __int8)(Address)&0x00FF;
	m_ASCPmsg.dibytes.parms[1] = (unsigned __int8)(Address>>8)&0x00FF;
	m_ASCPmsg.dibytes.parms[2] = (unsigned __int8)D0&0x00FF;
	m_ASCPmsg.dibytes.parms[3] = (unsigned __int8)D1&0x00FF;
	m_ASCPmsg.dibytes.parms[4] = (unsigned __int8)D2&0x00FF;
	m_ASCPmsg.dibytes.parms[5] = (unsigned __int8)D3&0x00FF;
	m_ASCPmsg.dibytes.parms[6] = (unsigned __int8)D4&0x00FF;
	sdr14USBio::SendASCPMsg( &m_ASCPmsg);
}


//////////////////////////////////////////////////////////////////////
//  Method called to start SDR-14 capturing data
//////////////////////////////////////////////////////////////////////
void sdr14ctrl::Start(short Blocks, short Mode) 
{
	if(!sdr14USBio::m_Running && sdr14USBio::m_USBPortOpen )	//ignor if already running or usb port closed
	{
		m_CaptureMode = Mode;
		if(m_InputSource&0x80) //if using AD6620 downconverter
		{
			m_SampleRate = m_ClockFrequency/sdr14USBio::Load6620Params(m_IFfilter);
		}
		else	//is raw A/D sampling mode
		{
			m_SampleRate = m_ClockFrequency;
			sdr14USBio::m_Last6620FilterLoaded = -1;
			SetModifiedFlag(TRUE);	//make sure NCO reloads next time used
		}
		if( IsModified() )	//if properties changed reload the world
		{
			SetIFgain(m_IFgain);
			SetHFgain(m_HFgain);
			SetClockFrequency(m_ClockFrequency); 
			SetNCOfrequency(m_NCOfrequency); 
			SetModifiedFlag(FALSE);
		}
		if( (m_CaptureMode == CAPTURE_MODE_CONTIGUOUS) &&
					(m_SampleRate>MAX_CONTIGUOUS_FREQ) )
		{	//can't run contiguous so make continuous
			m_CaptureMode = CAPTURE_MODE_CONTINUOUS;
		}
		// calcualte SDR-14 data watchdog timer value for data0 Acking
		if(m_SampleRate<=MAX_CONTIGUOUS_FREQ) m_DataAckCounterLoadVal = m_SampleRate/DATA_IQ_PAIRS;
		else m_DataAckCounterLoadVal = 60;
		m_DataAckCounter = m_DataAckCounterLoadVal;
		m_ADOverload = FALSE;
		m_IQOffset = CalcOffsetIQ();	//make sure IQOffset is up to date

		SetupDataBuffers(Blocks);	//create data fifo buffers
		//send start command to SDR-14
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;
		m_ASCPmsg.bytes.parms[1] = RX_STATE_ON;
		m_ASCPmsg.bytes.parms[2] = (unsigned __int8)m_CaptureMode;
		m_ASCPmsg.bytes.parms[3] = (unsigned __int8)Blocks;
		sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	}
}

//////////////////////////////////////////////////////////////////////
//  Method called to stop the SDR-14 data capturing
//////////////////////////////////////////////////////////////////////
void sdr14ctrl::Stop() 
{
	if(sdr14USBio::m_Running && sdr14USBio::m_USBPortOpen )	//ignor if alreay stopped
	{
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;	//active rx channel
		m_ASCPmsg.bytes.parms[1] = RX_STATE_IDLE;
		m_ASCPmsg.bytes.parms[2] = 0;
		m_ASCPmsg.bytes.parms[3] = 0;
		sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	}
}


void sdr14ctrl::ErrorStop()
{
	if(sdr14USBio::m_Running && sdr14USBio::m_USBPortOpen )	//ignor if alreay stopped
	{
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;	//active rx channel
		m_ASCPmsg.bytes.parms[1] = RX_STATE_IDLE;
		m_ASCPmsg.bytes.parms[2] = 0;
		m_ASCPmsg.bytes.parms[3] = 0;
		sdr14USBio::SendASCPMsg( &m_ASCPmsg);
	}
}

