#ifndef SDR14_FTD2XX_H
#define SDR14_FTD2XX_H

// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the FTD2XX_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// FTD2XX_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.

#ifndef FT14_SUPPORT
#define FT14_SUPPORT

#pragma pack(push)
#pragma pack(1)

typedef PVOID	FT14_HANDLE;
typedef ULONG	FT14_STATUS;

//
// Device status
//
enum {
    FT14_OK,
    FT14_INVALID_HANDLE,
    FT14_DEVICE_NOT_FOUND,
    FT14_DEVICE_NOT_OPENED,
    FT14_IO_ERROR,
    FT14_INSUFFICIENT_RESOURCES,
    FT14_INVALID_PARAMETER,
    FT14_INVALID_BAUD_RATE,

    FT14_DEVICE_NOT_OPENED_FOR_ERASE,
    FT14_DEVICE_NOT_OPENED_FOR_WRITE,
    FT14_FAILED_TO_WRITE_DEVICE,
    FT14_EEPROM_READ_FAILED,
    FT14_EEPROM_WRITE_FAILED,
    FT14_EEPROM_ERASE_FAILED,
	FT14_EEPROM_NOT_PRESENT,
	FT14_EEPROM_NOT_PROGRAMMED,
	FT14_INVALID_ARGS,
	FT14_NOT_SUPPORTED,
	FT14_OTHER_ERROR
};


#define FT14_SUCCESS(status) ((status) == FT14_OK)

//
// FT14_OpenEx Flags
//

#define FT14_OPEN_BY_SERIAL_NUMBER    1
#define FT14_OPEN_BY_DESCRIPTION      2
#define FT14_OPEN_BY_LOCATION			4

//
// FT14_ListDevices Flags (used in conjunction with FT14_OpenEx Flags
//

#define FT14_LIST_NUMBER_ONLY		0x80000000
#define FT14_LIST_BY_INDEX			0x40000000
#define FT14_LIST_ALL				0x20000000

#define FT14_LIST_MASK (FT14_LIST_NUMBER_ONLY|FT14_LIST_BY_INDEX|FT14_LIST_ALL)

//
// Baud Rates
//

#define FT14_BAUD_300		300
#define FT14_BAUD_600		600
#define FT14_BAUD_1200		1200
#define FT14_BAUD_2400		2400
#define FT14_BAUD_4800		4800
#define FT14_BAUD_9600		9600
#define FT14_BAUD_14400		14400
#define FT14_BAUD_19200		19200
#define FT14_BAUD_38400		38400
#define FT14_BAUD_57600		57600
#define FT14_BAUD_115200	115200
#define FT14_BAUD_230400	230400
#define FT14_BAUD_460800	460800
#define FT14_BAUD_921600	921600

//
// Word Lengths
//

#define FT14_BITS_8			(UCHAR) 8
#define FT14_BITS_7			(UCHAR) 7
#define FT14_BITS_6			(UCHAR) 6
#define FT14_BITS_5			(UCHAR) 5

//
// Stop Bits
//

#define FT14_STOP_BITS_1	(UCHAR) 0
#define FT14_STOP_BITS_1_5	(UCHAR) 1
#define FT14_STOP_BITS_2	(UCHAR) 2

//
// Parity
//

#define FT14_PARITY_NONE	(UCHAR) 0
#define FT14_PARITY_ODD		(UCHAR) 1
#define FT14_PARITY_EVEN	(UCHAR) 2
#define FT14_PARITY_MARK	(UCHAR) 3
#define FT14_PARITY_SPACE	(UCHAR) 4

//
// Flow Control
//

#define FT14_FLOW_NONE        0x0000
#define FT14_FLOW_RTS_CTS     0x0100
#define FT14_FLOW_DTR_DSR     0x0200
#define FT14_FLOW_XON_XOFF    0x0400

//
// Purge rx and tx buffers
//
#define FT14_PURGE_RX         1
#define FT14_PURGE_TX         2

//
// Events
//

typedef void (*PFT14_EVENT_HANDLER)(DWORD,DWORD);

#define FT14_EVENT_RXCHAR		    1
#define FT14_EVENT_MODEM_STATUS   2

//
// Timeouts
//

#define FT14_DEFAULT_RX_TIMEOUT   300
#define FT14_DEFAULT_TX_TIMEOUT   300

//
// Device types
//

typedef ULONG	FT14_DEVICE;

enum {
    FT14_DEVICE_BM,
    FT14_DEVICE_AM,
    FT14_DEVICE_100AX,
    FT14_DEVICE_UNKNOWN,
    FT14_DEVICE_2232C,
    FT14_DEVICE_232R
};


//
// structure to hold program data for FT14_Program function
//
typedef struct FT14_program_data {

	DWORD Signature1;			// Header - must be 0x00000000 
	DWORD Signature2;			// Header - must be 0xffffffff
	DWORD Version;				// Header - FT14_PROGRAM_DATA version
								//          0 = original
	                            //          1 = FT2232C extensions
								//			2 = FT232R extensions

	WORD VendorId;				// 0x0403
	WORD ProductId;				// 0x6001
	char *Manufacturer;			// "FTDI"
	char *ManufacturerId;		// "FT"
	char *Description;			// "USB HS Serial Converter"
	char *SerialNumber;			// "FT000001" if fixed, or NULL
	WORD MaxPower;				// 0 < MaxPower <= 500
	WORD PnP;					// 0 = disabled, 1 = enabled
	WORD SelfPowered;			// 0 = bus powered, 1 = self powered
	WORD RemoteWakeup;			// 0 = not capable, 1 = capable
	//
	// Rev4 extensions
	//
	UCHAR Rev4;					// non-zero if Rev4 chip, zero otherwise
	UCHAR IsoIn;				// non-zero if in endpoint is isochronous
	UCHAR IsoOut;				// non-zero if out endpoint is isochronous
	UCHAR PullDownEnable;		// non-zero if pull down enabled
	UCHAR SerNumEnable;			// non-zero if serial number to be used
	UCHAR USBVersionEnable;		// non-zero if chip uses USBVersion
	WORD USBVersion;			// BCD (0x0200 => USB2)
	//
	// FT2232C extensions
	//
	UCHAR Rev5;					// non-zero if Rev5 chip, zero otherwise
	UCHAR IsoInA;				// non-zero if in endpoint is isochronous
	UCHAR IsoInB;				// non-zero if in endpoint is isochronous
	UCHAR IsoOutA;				// non-zero if out endpoint is isochronous
	UCHAR IsoOutB;				// non-zero if out endpoint is isochronous
	UCHAR PullDownEnable5;		// non-zero if pull down enabled
	UCHAR SerNumEnable5;		// non-zero if serial number to be used
	UCHAR USBVersionEnable5;	// non-zero if chip uses USBVersion
	WORD USBVersion5;			// BCD (0x0200 => USB2)
	UCHAR AIsHighCurrent;		// non-zero if interface is high current
	UCHAR BIsHighCurrent;		// non-zero if interface is high current
	UCHAR IFAIsFifo;			// non-zero if interface is 245 FIFO
	UCHAR IFAIsFifoTar;			// non-zero if interface is 245 FIFO CPU target
	UCHAR IFAIsFastSer;			// non-zero if interface is Fast serial
	UCHAR AIsVCP;				// non-zero if interface is to use VCP drivers
	UCHAR IFBIsFifo;			// non-zero if interface is 245 FIFO
	UCHAR IFBIsFifoTar;			// non-zero if interface is 245 FIFO CPU target
	UCHAR IFBIsFastSer;			// non-zero if interface is Fast serial
	UCHAR BIsVCP;				// non-zero if interface is to use VCP drivers
	//
	// FT232R extensions
	//
	UCHAR UseExtOsc;			// Use External Oscillator
	UCHAR HighDriveIOs;			// High Drive I/Os
	UCHAR EndpointSize;			// Endpoint size

	UCHAR PullDownEnableR;		// non-zero if pull down enabled
	UCHAR SerNumEnableR;		// non-zero if serial number to be used

	UCHAR InvertTXD;			// non-zero if invert TXD
	UCHAR InvertRXD;			// non-zero if invert RXD
	UCHAR InvertRTS;			// non-zero if invert RTS
	UCHAR InvertCTS;			// non-zero if invert CTS
	UCHAR InvertDTR;			// non-zero if invert DTR
	UCHAR InvertDSR;			// non-zero if invert DSR
	UCHAR InvertDCD;			// non-zero if invert DCD
	UCHAR InvertRI;				// non-zero if invert RI

	UCHAR Cbus0;				// Cbus Mux control
	UCHAR Cbus1;				// Cbus Mux control
	UCHAR Cbus2;				// Cbus Mux control
	UCHAR Cbus3;				// Cbus Mux control
	UCHAR Cbus4;				// Cbus Mux control

	UCHAR RIsVCP;				// non-zero if using VCP drivers

} FT14_PROGRAM_DATA, *PFT14_PROGRAM_DATA;


#endif //FT14_SUPPORT

#pragma pack(pop)
#endif  /* SDR14_FTD2XX_H */

