////////////////////////////////////////////////////
////////////////////////////////////////////////////
// !!! this is not public domain !!!
// You can do what you want if it never hurt bonito.
// Otherwise, you will need the consent of Bonito.	
////////////////////////////////////////////////////
// RadioJet Open Source Project RJ10Mini
// www.bonito.net/radiojet/opensource
////////////////////////////////////////////////////
////////////////////////////////////////////////////

#include "stdafx.h"
#include "../GlobalTools.h"
#include "MyDCBitmap.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


MyDCBitmap::MyDCBitmap()
{

	m_bpl = NULL;		//byte per lines

	m_hbm = NULL;
	o_hbm = NULL;
	m_adr = NULL;
	m_hWnd = NULL;
	m_hdcScr = NULL;
	m_hdcDst = NULL;
	memset(&m_dib, 0, sizeof(CDIB_BITMAP));
	m_adr = NULL;
}

MyDCBitmap::~MyDCBitmap()
{
	DeleteCompatibleDC();
	DeleteBitmap();

}

int MyDCBitmap::Copy(MyDCBitmap& cib)
{
	DeleteBitmap();

	m_width	=cib.m_width;
	m_height=cib.m_height;
	m_hWnd	=cib.m_hWnd;
	m_bpl	=cib.m_bpl;

	// Fill in the BITMAPINFO structure with values
	memcpy(&m_dib,&cib.m_dib,sizeof(CDIB_BITMAP));

	HDC hdc = GetDC(m_hWnd);
	m_hbm = CreateDIBSection(hdc,(BITMAPINFO*)&m_dib,DIB_PAL_COLORS,(void **)&m_adr,NULL,(DWORD)NULL); 
	memcpy(m_adr,cib.m_adr,cib.m_dib.bmiHead.biSizeImage);
	ReleaseDC(m_hWnd, hdc);
	
	return TRUE;
}

int MyDCBitmap::CreateBitmap(HWND hWnd, int width, int height,COLORREF fillCollor)
{
	DeleteCompatibleDC();
	DeleteBitmap();

	m_hWnd = hWnd;
	m_width=width;
	m_height=height;

	m_bpl=width*3;
	m_bpl+=3; m_bpl&=~3;

	// Fill in the BITMAPINFO structure with values
	m_dib.bmiHead.biSize = sizeof(BITMAPINFOHEADER);
	m_dib.bmiHead.biWidth = width;
	m_dib.bmiHead.biHeight = -height;
	m_dib.bmiHead.biPlanes = 1;
	m_dib.bmiHead.biBitCount = 24;
	m_dib.bmiHead.biCompression = BI_RGB;
	m_dib.bmiHead.biSizeImage = m_bpl*height;
	m_dib.bmiHead.biClrUsed = 0;
	m_dib.bmiHead.biClrImportant = 0;

	HDC hdc = GetDC(m_hWnd);
	m_hbm = CreateDIBSection(hdc,(BITMAPINFO*)&m_dib,DIB_PAL_COLORS,(void **)&m_adr,NULL,(DWORD)NULL); 
	ReleaseDC(m_hWnd, hdc);

	SelectCompatibleDC();
	::FillRect(m_hdcScr,CRect(0,0,width,height),CBrush(fillCollor)); 
	DeleteCompatibleDC();

	if(m_hbm && m_adr) return TRUE;
	return FALSE;
}


void MyDCBitmap::DeleteBitmap(void)
{
	if (m_hbm) 
		{
		HDC hdc = GetDC(m_hWnd);
		::DeleteObject(m_hbm);
		ReleaseDC(m_hWnd, hdc);
		}

	memset(&m_dib, 0, sizeof(CDIB_BITMAP));
	m_hbm = NULL;
	m_adr = NULL;
	m_hWnd = NULL;
}


BYTE *MyDCBitmap::GetMapAdress(int x, int y)
{
	if(!m_adr) return NULL;

	if(x<0) x=0;
	if(x>=m_width) x=m_width-1;
	if(y<0) y=0;
	if(y>=m_height) y=m_height-1;
	m_pnt = &m_adr[y*m_bpl + x*3];
	return m_pnt;
}

void MyDCBitmap::AddAreaBrightness(int xstart, int ystart, int xend, int yend, float factor)
{
	xend--; yend--;
	if(xstart<0) xstart=0;
	if(xstart>m_width) xstart=m_width;
	if(xend<xstart) return;
	if(xend>m_width) xend=m_width;
	if(ystart<0) ystart=0;
	if(ystart>m_height) ystart=m_height;
	if(yend<ystart) return;
	if(yend>m_height) yend=m_height;

	int x,y,z;
	float c;
	for(y=ystart; y<yend; y++)
		{
		for(x=xstart*3; x<xend*3; x++)
			{
			z=m_bpl*y + x;
			c=(float)m_adr[z]*factor; 
			if(c<0) c=0; if(c>255) c=255;
			m_adr[z]=(BYTE) c;
			}
		}
}

HDC MyDCBitmap::SelectCompatibleDC()
{
	m_hdcScr = CreateCompatibleDC(NULL) ;
	o_hbm = (HBITMAP)SelectObject(m_hdcScr, m_hbm);

	return m_hdcScr;
}

void MyDCBitmap::DeleteCompatibleDC()
{
	if(m_hdcScr)
		{
		SelectObject(m_hdcScr, o_hbm);
		DeleteDC(m_hdcScr);
		m_hdcScr=NULL;
		o_hbm=NULL;
		}
}


BOOL MyDCBitmap::BitBlt(HDC hdc, int x, int y, int nWidth, int nHeight, int xSrc, int ySrc, DWORD dwRop)
{
	if(!m_hbm) return FALSE;

//	GdiFlush();

	HDC hdcSrc = CreateCompatibleDC(hdc) ;
	HBITMAP oldhbm = (HBITMAP)SelectObject(hdcSrc, m_hbm);
	BOOL result = ::BitBlt(hdc, x, y, nWidth, nHeight,hdcSrc, xSrc, ySrc, dwRop);
	SelectObject(hdcSrc, oldhbm);
	DeleteDC(hdcSrc);

	return result;
}

